import datetime, yaml

import couchdbkit as couch
from couchdbkit.loaders import FileSystemDocsLoader

from vogeler.exceptions import VogelerPersistenceException

class SystemRecord(couch.Document):
    system_name = couch.StringProperty()
    created_at = couch.DateTimeProperty()
    updated_at = couch.DateTimeProperty()

class VogelerStore(object):

    def __init__(self, **kwargs):
        try:
            host, port = kwargs['host'], kwargs['port']
            db = kwargs['db']
            connection_string = "http://%s:%s" % (host, port)
            self.server = couch.Server(uri=connection_string)
            self.dbname = db
        except:
            raise

    def create_db(self, dbname=''):
        try:
            if dbname == '':
                dbname = self.dbname

            self.db = self.server.get_or_create_db(dbname)
            SystemRecord.set_db(self.db)
        except:
            raise

    def drop_db(self, dbname=''):
        try:
            if dbname == '':
                dbname = self.dbname

            self.server.delete_db(dbname)
        except:
            raise

    def use_db(self, dbname=''):
        try:
            if dbname == '':
                dbname = self.dbname

            self.db = self.server.get_or_create_db(dbname)
            SystemRecord.set_db(self.db)
        except:
            raise

    def create(self, node_name):
        try:
            node = SystemRecord.get_or_create(node_name)
            node.system_name = node_name
            node.created_at = datetime.datetime.utcnow()
            node.save()
        except:
            raise

    def get(self, node_name):
        try:
            node = SystemRecord.get(node_name)
            self.node = node
            return node
        except:
            raise

    def touch(self, node_name):
        try:
            node = SystemRecord.get(node_name)
            node.updated_at = datetime.datetime.utcnow()
            node.save()
        except:
            raise

    def update(self, node_name, key, value, datatype):
        node = SystemRecord.get_or_create(node_name)

        if datatype == 'output':
            v = [z.strip() for z in value.split("\n")]
            node[key] = v
        if datatype == 'pylist':
            v = value
            node[key] = v
        if datatype == 'pydict':
            v = value
            node[key] = v
        if datatype == 'yaml':
            v = yaml.load(value)
            node[key] = v
        if datatype == 'string':
            v = value
            node[key] = v
        if datatype == 'raw':
            v = value
            node[key] = v
        node.updated_at = datetime.datetime.utcnow()
        node.save()

    def load_views(self, lp):
        self.loadpath = lp
        try:
            print "Loading design docs from %s" % lp
            loader = FileSystemDocsLoader(self.loadpath)
            loader.sync(self.db, verbose=True)
            print "Design docs loaded"
            return 0
        except:
            raise VogelerPersistenceException("Document load path not found: %s" % lp)

# vim: set ts=4 et sw=4 sts=4 sta filetype=python :



import json

from vogeler.exceptions import VogelerRunnerException
from vogeler.messaging import amqp

class VogelerRunner(object):
    def __init__(self, destination, **kwargs):
        try:
            self.routing_key = destination
            self.ch = amqp.setup_amqp(kwargs['host'], kwargs['username'], kwargs['password'])
        except:
            raise VogelerRunnerException("Unable to connect to %s as %s" % (kwargs['host'], kwargs['username']) )

    def message(self, message, durable=True):
        print "Vogeler(Runner) is sending a message"
        try:
            msg = amqp.amqp.Message(json.dumps(message))
            if durable == True:
                msg.properties['deliver_mode'] = 2
            self.ch.basic_publish(msg, exchange=amqp.broadcast_exchange, routing_key=self.routing_key)
        except:
            raise VogelerRunnerException("Unable to publish message: %s" % message)

    def close(self):
        self.ch.close()

# vim: set ts=4 et sw=4 sts=4 sta filetype=python :



import json

from vogeler.exceptions import VogelerServerException
from vogeler.messaging import amqp

class VogelerServer(object):
    def __init__(self, callback_function=None, **kwargs):
        try:
            self.ch, self.queue = amqp.setup_server(kwargs['host'], kwargs['username'], kwargs['password'])
            self.callback_function = callback_function
        except:
            raise VogelerServerException("Unable to connect to messaging system on %s as %s" % (kwargs['host'], kwargs['username']))

    def callback(self, msg):
        message = json.loads(msg.body)
        if(self.callback_function):
            self.callback_function(message)

    def monitor(self):
        try:
            print "Vogeler(Server) is starting up"
            self.ch.basic_consume(self.queue, callback=self.callback, no_ack=True)
        except:
            raise VogelerServerException("Unable to consume queue")

        while self.ch.callbacks:
            self.ch.wait()

    def message(self, message, durable=True):
        print "Vogeler(Server) is sending a message"
        try:
            msg = amqp.amqp.Message(json.dumps(message))
            if durable == True:
                msg.properties['delivery_mode'] = 2
            self.ch.basic_publish(msg, exchange=amqp.broadcast_exchange)
        except:
            raise VogelerServerException("Unable to publish message to the queue")

    def close(self):
        self.ch.close()




class VogelerException(Exception):
    """Base class for all exceptions"""

class VogelerPersistenceException(VogelerException):
    """Exception for an error in the persistence engine"""

class VogelerServerException(VogelerException):
    """Exception for an error in the server component"""

class VogelerClientException(VogelerException):
    """Exception for an error in the client component"""

class VogelerRunnerException(VogelerException):
    """Exception for an error in the runner component"""

class VogelerMessagingException(VogelerException):
    """Exception for an error in the messaging component"""

class VogelerPluginException(VogelerException):
    """Exception for an error in the plugin component"""

class VogelerEncryptionException(VogelerException):
    """Exception for an error in the encryption component"""

# vim: set ts=4 et sw=4 sts=4 sta filetype=python :



from amqplib import client_0_8 as amqp
from platform import node

from vogeler.exceptions import VogelerException

vhost = "/vogeler"
master_exchange = "vogeler.master.in"
broadcast_exchange = "vogeler.broadcast.in"
client_id = node()

def setup_client(host='', username='', password=''):
    node_name = client_id
    client_queue = node_name
    try:
        # Get a channel
        ch = setup_amqp(host, username, password)
        # define our incoming and outgoing queues
        ch.queue_declare(node_name, durable=True, auto_delete=False)
        # bind our queues to the channel
        ## this is for broadcast messages from the Vogeler server
        ch.queue_bind(client_queue, broadcast_exchange, routing_key='broadcasts.*')
        ## this is for messages intended for us over the same topic exchange
        ch.queue_bind(client_queue, broadcast_exchange, routing_key=node_name)
    except:
        raise
    return ch, client_queue

def setup_server(host='', username='', password=''):
    server_queue = 'master.in'
    try:
        # Get a channel
        ch = setup_amqp(host, username, password)
        # Setup our exchanges
        ## broadcast exchange for clients to recieve messages
        ch.exchange_declare(broadcast_exchange, 'topic', durable=True, auto_delete=False)
        ## direct exchange for server to get messages from clients
        ch.exchange_declare(master_exchange, 'direct', durable=True, auto_delete=False)
        # Now our own queues
        ch.queue_declare(server_queue, durable=True, auto_delete=False)
        # And then we bind to our channel
        ch.queue_bind(server_queue, master_exchange)
    except:
        raise
    return ch, server_queue

def setup_amqp(phost, puserid, ppassword):
    try:
        conn = amqp.Connection(host=phost,
            userid=puserid,
            password=ppassword,
            virtual_host=vhost,
            insist=False)
        ch = conn.channel()
        ch.access_request(vhost, active=True, read=True, write=True)
    except:
        raise
    return ch

# vim: set ts=4 et sw=4 sts=4 sta filetype=python :



import sys

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

version = '0.8.7'

# Anyone know a better way to do python version conditionals in install_requires?
modules = ['amqplib', 'couchdbkit', 'pyyaml']

if sys.version_info < (2, 7):
    two_six = ['argparse', 'anyjson']
    modules.extend(two_six)

if __name__ == "__main__":

    setup(name='vogeler',
          version=version,
          description="Python-based CMDB",
          long_description=open('README.rst', 'r').read(),
          classifiers=[
              'Development Status :: 4 - Beta',
              'Environment :: Console',
              'Intended Audience :: System Administrators',
              'License :: OSI Approved :: GNU General Public License (GPL)',
              'Natural Language :: English',
              'Programming Language :: Python :: 2.6',
              'Programming Language :: Python :: 2.7',
              'Topic :: System :: Systems Administration',
              'Operating System :: POSIX :: Linux',
          ],
          keywords='cmdb',
          author='John E. Vincent',
          author_email='lusis.org+github.com@gmail.com',
          url='http://lusis.github.com/vogeler',
          packages=[
            "vogeler",
            "vogeler.db",
            "vogeler.queue",
          ],
          install_requires=modules,
          scripts=[
            "scripts/vogeler-client",
            "scripts/vogeler-runner",
            "scripts/vogeler-server",
          ],
          data_files=[
            ('/tmp/vogeler/plugins',
                ['etc/plugins/facter.cfg', 'etc/plugins/rpms.cfg', 'etc/plugins/ps.cfg'] ),
            ("/tmp/vogeler/_design/system_records/views/all",
                ['etc/_design/system_records/views/all/map.js'] ),
            ("/tmp/vogeler/_design/system_records/views/by_name",
                ['etc/_design/system_records/views/by_name/map.js'] ),
            ("/tmp/vogeler/_design/system_records/views/packages_by_host",
                ['etc/_design/system_records/views/packages_by_host/map.js'] )
            ],
          )
